;------------------------------------------------------------------------------
;-         ATMEL Microcontroller Software Support  -  ROUSSET  -
;------------------------------------------------------------------------------
; The software is delivered "AS IS" without warranty or condition of any
; kind, either express, implied or statutory. This includes without
; limitation any warranty or condition with respect to merchantability or
; fitness for any particular purpose, or against the infringements of
; intellectual property rights of others.
;-----------------------------------------------------------------------------
;- File source          : cstartup_boot.arm
;- Object               : Generic CStartup
;- Compilation flag     : None
;-
;- 1.0 23/10/02 	FB  : Creation ARM ADS
;------------------------------------------------------------------------------

;/* modified 2004-2007 by Dmitriy Cherkashin */
;/* email: divch@users.sourceforge.net, dch@ucrouter.ru */
;/* based on ATMEL AT91RM9200-BasicBoot, */
;/* AT91RM9200-BasicROM_Services and */
;/* BOOTROM code */
;/* MIT licence */

	INCLUDE   AT91RM9200.inc

;--------------------------------
;- ARM Core Mode and Status Bits
;--------------------------------
ARM_MODE_USER		EQU	0x10	; /* user mode */
ARM_MODE_FIQ		EQU	0x11	; /* FIQ mode */
ARM_MODE_IRQ		EQU	0x12	; /* IRQ mode */
ARM_MODE_SVC		EQU	0x13	; /* supervisor mode */ 
ARM_MODE_ABORT		EQU	0x17	; /* data abord or prefech  */
ARM_MODE_UNDEF		EQU	0x1B	; /* undefined instruction mode */
ARM_MODE_SYS		EQU	0x1F	; /* system mode */

I_BIT				EQU	0x80
F_BIT				EQU	0x40
T_BIT				EQU	0x20

IRQ_STACK_SIZE		EQU	0x100
FIQ_STACK_SIZE		EQU	0x20
ABT_STACK_SIZE		EQU	0x20
UND_STACK_SIZE		EQU	0x20
SVC_STACK_SIZE		EQU	0x20		
USER_STACK_SIZE		EQU	0x100	; - exectly - SYStem mode stack size  

;------------------------------------------------------------------------------
;- Stack Area Definition
;------------------------------------------------------------------------------
	    AREA	|C$$stack|, DATA

;-IRQ stack definition
AT91_IRQ_Stack_End   DCD     0x55AA55AA
		     SPACE  (IRQ_STACK_SIZE - 4)
AT91_IRQ_Stack_Begin EQU    (AT91_IRQ_Stack_End + (IRQ_STACK_SIZE - 4))

;-FIQ stack definition
AT91_FIQ_Stack_End   DCD     0x55AA55AA
		     SPACE  (FIQ_STACK_SIZE - 4)
AT91_FIQ_Stack_Begin EQU    (AT91_FIQ_Stack_End + (FIQ_STACK_SIZE - 4))

;-ABORT stack definition
AT91_ABT_Stack_End   DCD     0x55AA55AA
		     SPACE  (ABT_STACK_SIZE - 4)
AT91_ABT_Stack_Begin EQU    (AT91_ABT_Stack_End + (ABT_STACK_SIZE - 4))

;-UNDEF stack definition
AT91_UND_Stack_End   DCD     0x55AA55AA
		     SPACE  (UND_STACK_SIZE - 4)
AT91_UND_Stack_Begin EQU    (AT91_UND_Stack_End + (UND_STACK_SIZE - 4))

;-SVC stack definition
AT91_SVC_Stack_End   DCD     0x55AA55AA
		     SPACE  (SVC_STACK_SIZE - 4)
AT91_SVC_Stack_Begin EQU    (AT91_SVC_Stack_End + (SVC_STACK_SIZE - 4))

;- USER and SYSTEM stack definition
AT91_USER_Stack_End  DCD     0x55AA55AA
		     SPACE  (USER_STACK_SIZE - 4)
AT91_USER_Stack_Begin EQU   (AT91_USER_Stack_End + (USER_STACK_SIZE - 4))

	EXPORT AT91_IRQ_Stack_End
	EXPORT AT91_IRQ_Stack_Begin
	EXPORT AT91_FIQ_Stack_End
	EXPORT AT91_FIQ_Stack_Begin
	EXPORT AT91_ABT_Stack_End
	EXPORT AT91_ABT_Stack_Begin
	EXPORT AT91_UND_Stack_End
	EXPORT AT91_UND_Stack_Begin
	EXPORT AT91_SVC_Stack_End
	EXPORT AT91_SVC_Stack_Begin
	EXPORT AT91_USER_Stack_End
	EXPORT AT91_USER_Stack_Begin

;------------------------------------------------------------------------------
;- Area Definition
;------------------------------------------------------------------------------
;- Must be defined as function to put first in the code as it must be mapped
;- at offset 0 of the flash EBI_CSR0, ie. at address 0 before remap.
;------------------------------------------------------------------------------
	    AREA        reset, CODE, READONLY

;	IMPORT __use_no_semihosting_swi
;------------------------------------------------------------------------------
;- Define the entry point
;------------------------------------------------------------------------------

	EXPORT	__ENTRY
__ENTRY

	IMPORT AT91F_SpuriousHandler	; Spurious handler
	IMPORT AT91F_UndefHandler		; Undefined instruction handler
	IMPORT AT91F_SwiHandler			; SWI
	IMPORT AT91F_DataAbort			; data abort handler
	IMPORT AT91F_FetchAbort			; command prefetch abort handler
	IMPORT AT91F_IrqHandler			; default IRQ handler
	IMPORT AT91F_FiqHandler			; FIQ handler

;------------------------------------------------------------------------------
;- Exception vectors ( before Remap )
;------------------------------------------------------------------------------
;- These vectors are read at address 0.
;- They absolutely requires to be in relative addresssing mode in order to
;- guarantee a valid jump. For the moment, all are just looping (what may be
;- dangerous in a final system). If an exception occurs before remap, this
;- would result in an infinite loop.
;------------------------------------------------------------------------------
		B	InitReset			; reset
undefvec
		B	AT91F_UndefHandler	; Undefined Instruction
swivec
		B	AT91F_SwiHandler	; Software Interrupt
pabtvec
		B	AT91F_FetchAbort	; Prefetch Abort
dabtvec 
		B	AT91F_DataAbort		; Data Abort
rsvdvec
		B	rsvdvec				; reserved
irqvec
		ldr pc, [pc,#-0xF20]	; IRQ - read from AIC
fiqvec
		ldr	pc, [pc,#-0xF20]	; FIQ - read from AIC

;------------------------------------------------------------------------------
;- The reset handler
;------------------------------------------------------------------------------
InitReset
; disable IRQ and FIQ
	msr     CPSR_c, #ARM_MODE_SYS :OR: I_BIT:OR:F_BIT
;------------------------------------------------------------------------------
;-PMC initialisation : Enable the Main Oscillator
;------------------------------------------------------------------------------
;-Slow clock mode init
;-After reset, only the 32KHz oscillator is enabled.
;-The ARM7TDMI or ARM9 runs the first instructions at 32768Hz <=> Slow Clock.
;-The processor clock and master clock are enabled at slow clock.
;-All the peripheral clocks are disabled.
;-The main oscillator is disabled.
;------------------------------------------------------------------------------
	ldr	r1, =AT91C_BASE_PMC		; Get the PMC Base Address

;-Interrupt Disable Register
	ldr	r0, = 0xFFFFFFFF
	str	r0, [r1, #PMC_IDR]

;-System clock disable register	PMC_SCDR : disable all clocks except CPU clock
	ldr	r0, = 0xFFFFFFFE		; bit 0 - CPU clock
	str	r0, [r1, #PMC_SCDR]

;-Peripheral clock Disable register : disable all peripheral clocks
	ldr	r0, = 0xFFFFFFFC		;-bits 0,1 - unused
	str	r0, [r1, #PMC_PCDR]

;-Master Clock Register PMC_MCKR : Slow Clock is selected
	ldr	r0, = AT91C_PMC_CSS_SLOW_CLK :OR: AT91C_PMC_PRES_CLK
	str	r0, [r1, #PMC_MCKR]

;- Reading the PMC Status register to detect when the Master Clock is commuted
	mov		r4, #0x08			; bit 3 - MCKRDY master clock ready bit
	mov		r5, #0x100			; 
MCKR_Slow_Loop
	subs    r5,r5,#1
	beq		MCKR_Slow_Loop_End

	ldr     r3, [r1, #PMC_SR]
	and		r3, r4, r3
	cmp		r3, r4
	bne     MCKR_Slow_Loop		; branch if MCKRDY bit not set
MCKR_Slow_Loop_End
                	
;-System clock Enable register	PMC_SCER : Enable only processor clock
	ldr	r0, = 0x01
	str	r0, [r1, #PMC_SCER]
                	
;------------------------------------------------------------------------------
;-After reset,PLLs are disabled
;-But in case of a boot already started, PLLs are turned off
;-Can be cleared if the project is used for a boot execution
;------------------------------------------------------------------------------
	ldr	r1, = AT91C_BASE_CKGR	; Get the CKGR Base Address

;------------------------------------------------------------------------------
;-Enabling the Main Oscillator
;-Normally First instruction in PMC initialisation
;------------------------------------------------------------------------------
;-Main oscillator Enable register	APMC_MOR : Enable main oscillator , OSCOUNT = 0xFF
	ldr	r0, = AT91C_CKGR_MOSCEN :OR: AT91C_CKGR_OSCOUNT
	str	r0, [r1, #CKGR_MOR]

;wait main oscillator clock ready
	mov	r4, #0x10000			; bit 16 - MAINRDY - main clock ready bit
	mov	r5, #0x100
MCKR_MainOsc_Loop
	subs	r5, r5, #1
	beq	MCKR_MainOsc_End

	ldr		r3, [r1, #CKGR_MCFR]
	and     r3, r4, r3
	cmp     r3, r4
	bne     MCKR_MainOsc_Loop	; 
MCKR_MainOsc_End

	ldr	r1, = AT91C_BASE_CKGR	; Get the CKGR Base Address
;-Master Clock Register PMC_PLLAR : Turned off PLLA
	ldr	r0, = AT91C_CKGR_DIVA_0
	str	r0, [r1, #CKGR_PLLAR]

;-Master Clock Register PMC_PLLBR : Turned off PLLB
	ldr	r0, = AT91C_CKGR_DIVB_0
	str	r0, [r1, #CKGR_PLLBR]

;-Enable clock for system peripheral and PIOA to clear output PA22
	ldr	r1, =AT91C_BASE_PMC		; Get the PMC Base Address
; === DDD === REMOVE ===
;	ldr	r0,=0x06				; Enable System Peripheral clock and PIOA clock
	ldr	r0,=0x16				; Enable System Peripheral clock and PIOA clock
	str	r0,[r1,#PMC_PCER]		;

;------------------------------------------------------------------------------
;- Setup the stack for each mode
;------------------------------------------------------------------------------
;- The processor will remain in the last initialized mode.
;------------------------------------------------------------------------------

;- Load the stack base addresses
	add	r0, pc,#-(8+.-StackData); @ where to read values (relative)
	ldmia r0, {r1-r6}

;- Set up Supervisor Mode and set SVC Mode Stack
	msr	cpsr_c, #ARM_MODE_SVC :OR: I_BIT :OR: F_BIT
	bic	r1, r1, #3		; Insure word alignement
	mov	sp, r1			; Init stack SYS

;- Set up Interrupt Mode and set IRQ Mode Stack
	msr CPSR_c, #ARM_MODE_IRQ :OR: I_BIT :OR: F_BIT
	bic r2, r2, #3		; Insure word alignement
	mov sp, r2			; Init stack IRQ

;- Set up Fast Interrupt Mode and set FIQ Mode Stack
	msr CPSR_c,	#ARM_MODE_FIQ :OR: I_BIT :OR: F_BIT
	bic	r3, r3, #3		; Insure word alignement
	mov sp, r3			; Init stack FIQ

;- Set up Abort Mode and set Abort Mode Stack
	msr CPSR_c, #ARM_MODE_ABORT :OR: I_BIT :OR: F_BIT
	bic	r4, r4, #3		; Insure word alignement
	mov	sp, r4			; Init stack Abort

;- Set up Undefined Instruction Mode and set Undef Mode Stack
	msr	CPSR_c, #ARM_MODE_UNDEF :OR: I_BIT :OR: F_BIT
	bic	r5, r5, #3		; Insure word alignement
	mov sp, r5			; Init stack Undef

;- Set up user Mode and set Undef Mode Stack
	msr	CPSR_c, #ARM_MODE_SYS:OR:I_BIT:OR:F_BIT
	bic	r6, r6, #3		; Insure word alignement
	mov	sp, r6			; Init stack Undef

	b EndInitStack
	
StackData
	DCD	AT91_SVC_Stack_Begin
	DCD	AT91_IRQ_Stack_Begin
	DCD	AT91_FIQ_Stack_Begin
	DCD	AT91_ABT_Stack_Begin
	DCD	AT91_UND_Stack_Begin
	DCD	AT91_USER_Stack_Begin
EndInitStack

;- Add loop to compensate Main Oscillator startup time
	ldr	r0, =0x00000010
LoopOsc
	subs    r0, r0, #1
	bhi	LoopOsc


;----------------------------------------
; Read/modify/write CP15 control register
;----------------------------------------
	MRC	p15, 0, r0, c1, c0, 0 	; read cp15 control registre (cp15 r1) in r0
	ldr	r3, =0xC0000080      	; Reset bit :Little Endian end fast bus mode
	ldr	r4, =0xC0000000      	; Set bit :Asynchronous clock mode, Not Fast Bus
	BIC	r0, r0, r3
	ORR	r0, r0, r4
	MCR	p15, 0, r0, c1, c0, 0	; write r0 in cp15 control registre (cp15 r1)

;------------------------------------------------------------------------------
;- Reset the Interrupt Controller
;------------------------------------------------------------------------------
;- Normally, the code is executed only if a reset has been actually performed.
;- So, the AIC initialization resumes at setting up the default vectors.
;------------------------------------------------------------------------------
;- Load the AIC Base Address and the default handler addresses
	add	r0, pc,#-(8+.-AicData)	; @ where to read values (relative)
	ldmia	r0, {r2-r4}
	ldr	r1, =AT91C_BASE_AIC

; r1 = AIC_BASE
; r2 = FIQ handler
; r3 = IRQ handler
; r4 = Spurious handler

;- Setup the Spurious Vector
	str     r4, [r1, #AIC_SPU]	; r4 = spurious handler

;- Set up the default interrupt handler vectors
	add	r1, r1, #AIC_SVR
	str	r2, [r1]				; SVR[0] for FIQ
	mov	r0, #31					; counter

AIC_Svr_Loop
	str	r3, [r1, r0, LSL #2]	; SVRs for IRQs
	subs r0, r0, #1				; do not save FIQ
	beq AIC_Svr_Loop_End		; 0 => source vector reg loop end
	b 	AIC_Svr_Loop
AIC_Svr_Loop_End

; set SMR
	ldr	r1, =AT91C_BASE_AIC
	add	r1, r1, #AIC_SMR
	mov	r3, #0					; 0 - Low priority, 0 - Level Sensitive
	str	r3, [r1]				; SMR for IRQ 0
	mov	r0, #31					; counter

AIC_SMR_Loop
	str	r3, [r1, r0, LSL #2]	; SVRs for IRQs
	subs r0, r0, #1				; do not save FIQ
	beq	AIC_SMR_Loop_End
    b   AIC_SMR_Loop
AIC_SMR_Loop_End

;Perform 8 End Of Interrupt Command to make sure AIC will not Lock out nIRQ
	mov	r0, #8
	mov	r3, #0
AIC_EOICR_Loop
	ldr	 r1, =AT91C_BASE_AIC
	str	 r3, [r1, #AIC_EOICR]
	subs r0, r0, #1
	beq	 AIC_EOICR_Loop_End
	b    AIC_EOICR_Loop
AIC_EOICR_Loop_End

	mov	r3,#0xFFFFFFFF
	str	r3, [r1, #AIC_IDCR]
		
	b	EndInitAic

;------------------------------------------------------------------------------
;- Default Interrupt Handler
;------------------------------------------------------------------------------
AicData
	DCD	AT91F_FiqHandler		; FIQ handler
	DCD	AT91F_IrqHandler		; default IRQ handler
	DCD	AT91F_SpuriousHandler	; Spurious handler
EndInitAic

;
	ldr	r1, =AT91C_BASE_PIOA	; PIO_PER PIO enable register
	ldr	r0, =0x00400000			; PA22 - bit 22

PA22_PER_set
	str	r0, [r1, #PIO_PER]		; PER  PIO Enable Register	
	ldr	r2, [r1, #PIO_PSR]		; PER  PIO Enable Register
	and	r2, r0, r2
	cmp	r2, r0
	bne	PA22_PER_set

PA22_OER_set
	str	r0, [r1, #PIO_OER]		; OER output enavble register
	ldr	r2, [r1, #PIO_OSR]		; PER  PIO Enable Register
	and	r2, r0, r2
	cmp	r2, r0
	bne	PA22_OER_set

PA22_OWER_set
	str	r0, [r1, #PIO_OWER]		; OWER Output Write Enable Register
	ldr	r2, [r1, #PIO_OWSR]		; PER  PIO Enable Register
	and	r2, r0, r2
	cmp	r2, r0
	bne	PA22_OWER_set

	str	r0, [r1, #PIO_CODR]		; CODR Clear Output

	ldr	r2, [r1, #PIO_ODSR]		; PER  PIO Enable Register
	and	r2, r0, r2
	cmp	r2, r0
	bne	PA22_ODSR_cleared
PA22_ODSR_cleared

;------------------------------------------------------------------------------
;- Initialise C variables
;------------------------------------------------------------------------------
;- Following labels are automatically generated by the linker.
;- RO: Read-only = the code
;- RW: Read Write = the data pre-initialized and zero-initialized.
;- ZI: Zero-Initialized.
;- Pre-initialization values are located after the code area in the image.
;- Zero-initialized datas are mapped after the pre-initialized.
;- Note on the Data position :
;- If using the ARMSDT, when no -rw-base option is used for the linker, the 
;- data area is mapped after the code. You can map the data either in internal
;- SRAM ( -rw-base=0x200000 ) or in external SRAM ( -rw-base=0x2000000 ).
;- Note also that to improve the code density, the pre_initialized data must 
;- be limited to a minimum.
;------------------------------------------------------------------------------
	add	r2, pc,#-(8+.-CInitData)  ; @ where to read values (relative)
	ldmia	r2, {r0, r1, r3, r4}
	
	cmp	r0, r1					; Check that they are different
	beq	EndRW
LoopRW
	cmp	r1, r3					; Copy RW data
	ldrcc	r2, [r0], #4
	strcc	r2, [r1], #4
	bcc	LoopRW
EndRW

	mov	r2, #0
LoopZI
	cmp	r3, r4					; Zero init
	strcc   r2, [r3], #4
	bcc	LoopZI
 
	b	EndInitC
                
CInitData
	IMPORT	|Image$$RO$$Limit|	; End of ROM code (=start of RW data)
	IMPORT	|Image$$RW$$Base|	; Base of RAM to initialise
	IMPORT	|Image$$ZI$$Base|	; Base and limit of area
	IMPORT	|Image$$ZI$$Limit|	; Top of zero init segment
	
	DCD	    |Image$$RO$$Limit|	; End of ROM code (=start of ROM data)
	DCD     |Image$$RW$$Base|	; Base of RAM to initialise
	DCD     |Image$$ZI$$Base|	; Base and limit of area
	DCD	    |Image$$ZI$$Limit|	; Top of zero init segment
EndInitC

; /* call low level init function */
	IMPORT    AT91F_LowLevelInit

	ldr	r0, = AT91F_LowLevelInit
	mov	lr, pc
	bx	r0

;enable only IRQ's and not FIQ's
	msr     cpsr_c, #ARM_MODE_SYS:OR:F_BIT
  
;------------------------------------------------------------------------------
;- Branch on C code Main function
;------------------------------------------------------------------------------
	IMPORT      main
_main
__main
	EXPORT	_main
    	EXPORT	__main
	ldr	r0, =main
	mov	lr, pc
	bx	r0

;------------------------------------------------------------------------------
;- Loop for ever
;------------------------------------------------------------------------------
;- End of application. Normally, never occur.
;- Could jump on Software Reset ( B 0x0 ).
;------------------------------------------------------------------------------
End
	b	End
	
	
	
            END
